﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;

namespace Mcp2210
{
    public partial class Mcp2210Device
    {
        public enum Commands : byte
        {
            GetChipStatus = 0x10, // implemented, not tested
            TransferSpiCancel = 0x11, // implemented, tested
            GetEventsCount = 0x12, // implemented, tested
            GetGpioChipSettings = 0x20, // implemented, tested
            SetGpioChipSettings = 0x21, // implemented, tested
            SetGpioCurrentPinValue = 0x30,  // implemented, tested
            GetGpioCurrentPinValue = 0x31, // implemented, tested
            SetGpioCurrentPinDirection = 0x32, // implemented, tested
            GetGpioCurrentPinDirection = 0x33, // implemented, tested
            SetSpiTransferSettings = 0x40, // implemented, tested
            GetSpiTransferSettings = 0x41, // implemented, tested
            TransferSpiData = 0x42, //implemented, tested
            WriteEepromMemory = 0x51, // not implemented
            SetNvramSettings = 0x60, // not implemented
            GetNvramSettings = 0x61, // not implemented
            SendAccessPassword = 0x70, // not implemented
            RequestSpiBusRelease = 0x80, // not implemented
        }

        public enum InterruptMode : byte
        {
            NoCounting = 0,
            FallingEdges = 1,
            RisingEdge = 2,
            LowPulses = 3,
            HighPulses = 4,
        }

        public enum SpiBusOwner : byte
        {
            NoOwner = 0,
            UsbBridge = 1,
            ExternalMaster = 2,
        }

        public enum GpioFunction : byte
        {
            Gpio = 0,
            ChipSelect = 1,
            DedicatedFunction = 2,
        }

        public enum SpiMode : byte
        {
            Mode0 = 0, // CPOL = 0, CPHA = 0 (clock idle low, data on first edge)
            Mode1 = 1, // CPOL = 0, CPHA = 1 (clock idle low, data on second edge)
            Mode2 = 2, // CPOL = 1, CPHA = 0 (clock idle high, data on second edge)
            Mode3 = 3, // CPOL = 1, CPHA = 1 (clock idle high, data on second edge)
        }

        [StructLayout(LayoutKind.Sequential, Size = 64, Pack = 1)]
        private struct ChipStatusCmd
        {
            public Commands cmdCode; // 0x10
            private byte dnc0;
            private byte dnc1;
        }

        [StructLayout(LayoutKind.Sequential, Size = 64, Pack = 1)]
        public struct ChipStatusResp
        {
            public Commands cmdCode; // 0x10
            public byte success;
            public byte spiBusReleaseRequest;
            public SpiBusOwner spiBusCurrentOwner;
            public byte passwordAttempts;
            public byte passwordGuessed;
        }

        [StructLayout(LayoutKind.Sequential, Size = 64, Pack = 1)]
        private struct EventCounterCmd
        {
            public Commands cmdCode; // 0x12
            public byte noReset;
        }

        [StructLayout(LayoutKind.Sequential, Size = 64, Pack = 1)]
        private struct EventCounterResp
        {
            public Commands cmdCode; // 0x12
            public byte success;
            private byte dnc0;
            private byte dnc1;
            public ushort eventCounter;
        }



        [StructLayout(LayoutKind.Sequential, Size = 64, Pack = 1)]
        internal struct SpiTransferSettings
        {
            public Commands cmdCode; // 0x40
            public byte success;
            private byte dnc1;
            private byte dnc2;
            public UInt32 bitrate;
            public ushort idleCsValues;
            public ushort activeCsValues;
            public ushort delayCsToData;
            public ushort delayDataToCs;
            public ushort delayDataToData;
            public ushort bytesPerTransfer;
            public SpiMode spiMode;
        }

        [StructLayout(LayoutKind.Sequential, Size = 64, Pack = 1)]
        private struct GetSpiTransferSettingsReq
        {
            public Commands cmdCode; // 0x41
            private byte dnc0;
            private byte dnc1;
        }

        [StructLayout(LayoutKind.Sequential, Size = 64, Pack = 1)]
        private struct GetGpioSettingsReq
        {
            public Commands cmdCode; // 0x20
            private byte dnc0;
            private byte dnc1;
        }

        [StructLayout(LayoutKind.Sequential, Size = 64, Pack = 1)]
        internal struct GpioSettings
        {
            public Commands cmdCode; // 0x20 for get resp, 0x21 for set cmd
            public byte success;
            private byte dnc0;
            private byte dnc1;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 9)]
            public GpioFunction[] gpioFunctions;
            public ushort gpioValue;
            public ushort gpioDirection;
            public byte otherChipSettings;
            public byte chipParmAccessControl;
        }

        [StructLayout(LayoutKind.Sequential, Size = 64, Pack = 1)]
        private struct SetGpioSettingsResp
        {
            public Commands cmdCode; // 0x21
            public byte success;
            private byte dnc0;
        }

        [StructLayout(LayoutKind.Sequential, Size = 64, Pack = 1)]
        private struct TransferSpiDataCmd
        {
            public Commands cmdCode; // 0x42
            public byte transmitLength;
            private byte dnc0;
            private byte dnc1;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 60)]
            public byte[] data;
        }

        [StructLayout(LayoutKind.Sequential, Size = 64, Pack = 1)]
        private struct TransferSpiDataResp
        {
            public Commands cmdCode; // 0x41
            public byte respCode;
            public byte receiveLength; // only for respCode == 0x00
            public byte engineStatus; // only for respCode == 0x00
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 60)]
            public byte[] data;
        }

        [StructLayout(LayoutKind.Sequential, Size = 64, Pack = 1)]
        private struct AnyCmd
        {
            public Commands cmdCode;
        }

        [StructLayout(LayoutKind.Sequential, Size = 64, Pack = 1)]
        public struct CancelSpiResp
        {
            public Commands cmdCode;
            public byte success;
            public byte releaseStatus;
            public byte spiOwner;
            public byte passwordTriedAccesses;
            public byte passwordGuessed;
        }

        [StructLayout(LayoutKind.Sequential, Size = 64, Pack = 1)]
        public struct GpioValues
        {
            public Commands cmdCode;
            public byte success;
            private byte dnc0;
            private byte dnc1;
            public ushort pinValues;
        }
    }
}
