﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Mcp2221
{
    public abstract class Mcp2221Gpio
    {
        protected Mcp2221Device device;
        protected int index;
        public byte _functionValue;

        public byte functionValue
        {
            get { return _functionValue; }
            set { _functionValue = value; }
        }

        private bool directionOutput;
        private bool iovalue;

        public bool DirectionOutput
        {
            get
            {
                device._gpioGet();
                return directionOutput;
            }
            set
            {
                directionOutput = value;
                device._gpioSet();
            }
        }

        public bool Value
        {
            get
            {
                device._gpioGet();
                return iovalue;
            }
            set
            {
                if (directionOutput == true)
                {
                    this.iovalue = value;
                    device._gpioSet();
                }
            }
        }

        protected Mcp2221Gpio(Mcp2221Device device)
        {
            this.device = device;
        }

        public Mcp2221Device.GpioSetOutValuesElement dev_SetGpioOutputValues()
        {
            Mcp2221Device.GpioSetOutValuesElement result = new Mcp2221Device.GpioSetOutValuesElement();
            result.AlterGpioOutput = 1; // alter value
            if (iovalue == true)
                result.GpioOutput = 1;

            result.AlterGpioDirection = 1; // alter direction
            if (directionOutput == false)
                result.GpioDirection = 1;

            return result;
        }

        public void _dev_GetGpioOutputValues(Mcp2221Device.GpioGetValuesRespElement data)
        {
            
            if (data.PinValue == 0)
                iovalue = false;
            else if (data.PinValue == 1)
                iovalue = true;

            if (data.PinDirection == 0)
                directionOutput = true;
            else if (data.PinDirection == 1)
                directionOutput = false;

        }

        public byte _dev_SetSramSettings()
        {
            byte result = 0;
            if (iovalue == true)
                result |= (1 << 4);

            if (directionOutput == false)
                result |= (1 << 3);

            result |= functionValue;

            return result;
        }

        public void _dev_GetSramSettings(byte data)
        {
            directionOutput = ((data & (1 << 3)) == 0);
            if(directionOutput == true)
            {
                iovalue = ((data & (1 << 4)) == 1);
            }
            functionValue = (byte)(data & 0x07);
        }
    }

    public class Mcp2221Gpio0 : Mcp2221Gpio
    {
        public enum Functions : byte
        {
            Gpio = 0,
            Suspend = 1,
            LedUartRx = 2,
        }

        public Functions Function
        {
            get
            {
                device._gpioGetfunc();
                return (Functions)functionValue;
            }
            set
            {
                functionValue = (byte)value;
                device._gpioSetfunc();
            }
        }

        public Mcp2221Gpio0(Mcp2221Device device)
            : base(device)
        {
            index = 0;
        }
    }

    public class Mcp2221Gpio1 : Mcp2221Gpio
    {
        public enum Functions : byte
        {
            Gpio = 0,
            ClockOut = 1,
            Adc1 = 2,
            LedUartTx = 3,
            InterruptDetection = 4,
        }

        public ushort AdcValue
        {
            get
            {
                return device._adcGet(index);
            }
        }

        public Functions Function
        {
            get
            {
                device._gpioGetfunc();
                return (Functions)functionValue;
            }
            set
            {
                functionValue = (byte)value;
                device._gpioSetfunc();
            }
        }

        public Mcp2221Gpio1(Mcp2221Device device)
            : base(device)
        {
            index = 1;
        }
    }

    public class Mcp2221Gpio2 : Mcp2221Gpio
    {
        public enum Functions : byte
        {
            Gpio = 0,
            UsbConfigured = 1,
            Adc2 = 2,
            Dac1 = 3,
        }

        public ushort AdcValue
        {
            get
            {
                return device._adcGet(index);
            }
        }

        public byte DacValue
        {
            get
            {
                return device._dacGet(index);
            }
            set
            {
                device._dacSet(index, value);
            }
        }

        public Functions Function
        {
            get
            {
                device._gpioGetfunc();
                return (Functions)functionValue;
            }
            set
            {
                functionValue = (byte)value;
                device._gpioSetfunc();
            }
        }

        public Mcp2221Gpio2(Mcp2221Device device)
            : base(device)
        {
            index = 2;
        }

    }

    public class Mcp2221Gpio3 : Mcp2221Gpio
    {
        public enum Functions : byte
        {
            Gpio = 0,
            LedI2c = 1,
            Adc3 = 2,
            Dac2 = 3,
        }

        public ushort AdcValue
        {
            get
            {
                return device._adcGet(index);
            }
        }

        public byte DacValue
        {
            get
            {
                return device._dacGet(index);
            }
            set
            {
                device._dacSet(index, value);
            }
        }

        public Functions Function
        {
            get
            {
                device._gpioGetfunc();
                return (Functions)functionValue;
            }
            set
            {
                byte tmp = (byte)value;
                functionValue = tmp;
                device._gpioSetfunc();
            }
        }

        public Mcp2221Gpio3(Mcp2221Device device)
            : base(device)
        {
            index = 3;
        }
    }
}
